@@ -1,13 +1,13 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
 use glib::translate::*;
-
 use gst::subclass::prelude::*;
 
-use crate::prelude::*;
-use crate::video_codec_state::{Readable, VideoCodecState};
-use crate::VideoCodecFrame;
-use crate::VideoDecoder;
+use crate::{
+    prelude::*,
+    video_codec_state::{Readable, VideoCodecState},
+    VideoCodecFrame, VideoDecoder,
+};
 
 pub trait VideoDecoderImpl: VideoDecoderImplExt + ElementImpl {
     fn open(&self) -> Result<(), gst::ErrorMessage> {
@@ -806,7 +806,7 @@ unsafe extern "C" fn video_decoder_getcaps<T: VideoDecoderImpl>(
                 .as_ref(),
         )
     })
-    .to_glib_full()
+    .into_glib_ptr()
 }
 
 unsafe extern "C" fn video_decoder_sink_event<T: VideoDecoderImpl>(
