@@ -1,6 +1,7 @@
+use glib::{object::IsA, translate::*};
+use gst::prelude::*;
+
 use crate::auto::VideoAggregator;
-use glib::object::IsA;
-use glib::translate::*;
 
 pub trait VideoAggregatorExtManual: 'static {
     fn video_info(&self) -> Option<crate::VideoInfo>;
@@ -10,7 +11,7 @@ impl<O: IsA<VideoAggregator>> VideoAggregatorExtManual for O {
     fn video_info(&self) -> Option<crate::VideoInfo> {
         unsafe {
             let ptr = self.as_ptr() as *mut ffi::GstVideoAggregator;
-            let _guard = crate::utils::MutexGuard::lock(&(*(ptr as *mut gst::ffi::GstObject)).lock);
+            let _guard = self.as_ref().object_lock();
 
             let info = &(*ptr).info;
 
