@@ -1,17 +1,17 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use crate::utils::HasStreamLock;
-use crate::video_codec_state::{InNegotiation, Readable, VideoCodecState, VideoCodecStateContext};
-use crate::VideoCodecFrame;
-use crate::VideoDecoder;
-use crate::VideoFormat;
+use std::{mem, ptr};
+
+use glib::{prelude::*, translate::*};
+
 #[cfg(any(feature = "v1_16", feature = "dox"))]
 #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_16")))]
 use crate::VideoInterlaceMode;
-use glib::prelude::*;
-use glib::translate::*;
-use std::mem;
-use std::ptr;
+use crate::{
+    utils::HasStreamLock,
+    video_codec_state::{InNegotiation, Readable, VideoCodecState, VideoCodecStateContext},
+    VideoCodecFrame, VideoDecoder, VideoFormat,
+};
 
 extern "C" {
     fn _gst_video_decoder_error(
@@ -45,26 +45,6 @@ pub trait VideoDecoderExtManual: 'static {
     #[doc(alias = "get_allocator")]
     fn allocator(&self) -> (Option<gst::Allocator>, gst::AllocationParams);
 
-    fn finish_frame(&self, frame: VideoCodecFrame) -> Result<gst::FlowSuccess, gst::FlowError>;
-    fn release_frame(&self, frame: VideoCodecFrame);
-    fn drop_frame(&self, frame: VideoCodecFrame) -> Result<gst::FlowSuccess, gst::FlowError>;
-
-    #[cfg(any(feature = "v1_20", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
-    #[doc(alias = "gst_video_decoder_finish_subframe")]
-    fn finish_subframe(&self, frame: &VideoCodecFrame) -> Result<gst::FlowSuccess, gst::FlowError>;
-    #[cfg(any(feature = "v1_20", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
-    #[doc(alias = "gst_video_decoder_drop_subframe")]
-    fn drop_subframe(&self, frame: &VideoCodecFrame) -> Result<gst::FlowSuccess, gst::FlowError>;
-    #[cfg(any(feature = "v1_20", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
-    #[doc(alias = "gst_video_decoder_have_last_subframe")]
-    fn have_last_subframe(
-        &self,
-        frame: &VideoCodecFrame,
-    ) -> Result<gst::FlowSuccess, gst::FlowError>;
-
     #[doc(alias = "get_latency")]
     fn latency(&self) -> (gst::ClockTime, Option<gst::ClockTime>);
     fn set_latency(
@@ -136,43 +116,13 @@ impl<O: IsA<VideoDecoder>> VideoDecoderExtManual for O {
     fn allocator(&self) -> (Option<gst::Allocator>, gst::AllocationParams) {
         unsafe {
             let mut allocator = ptr::null_mut();
-            let mut params = mem::zeroed();
+            let mut params = mem::MaybeUninit::uninit();
             ffi::gst_video_decoder_get_allocator(
                 self.as_ref().to_glib_none().0,
                 &mut allocator,
-                &mut params,
+                params.as_mut_ptr(),
             );
-            (from_glib_full(allocator), params.into())
-        }
-    }
-
-    #[doc(alias = "gst_video_decoder_finish_frame")]
-    fn finish_frame(&self, frame: VideoCodecFrame) -> Result<gst::FlowSuccess, gst::FlowError> {
-        unsafe {
-            try_from_glib(ffi::gst_video_decoder_finish_frame(
-                self.as_ref().to_glib_none().0,
-                frame.into_glib_ptr(),
-            ))
-        }
-    }
-
-    #[doc(alias = "gst_video_decoder_release_frame")]
-    fn release_frame(&self, frame: VideoCodecFrame) {
-        unsafe {
-            ffi::gst_video_decoder_release_frame(
-                self.as_ref().to_glib_none().0,
-                frame.into_glib_ptr(),
-            )
-        }
-    }
-
-    #[doc(alias = "gst_video_decoder_drop_frame")]
-    fn drop_frame(&self, frame: VideoCodecFrame) -> Result<gst::FlowSuccess, gst::FlowError> {
-        unsafe {
-            try_from_glib(ffi::gst_video_decoder_drop_frame(
-                self.as_ref().to_glib_none().0,
-                frame.into_glib_ptr(),
-            ))
+            (from_glib_full(allocator), params.assume_init().into())
         }
     }
 
@@ -374,45 +324,6 @@ impl<O: IsA<VideoDecoder>> VideoDecoderExtManual for O {
         }
     }
 
-    #[cfg(any(feature = "v1_20", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
-    #[doc(alias = "gst_video_decoder_finish_subframe")]
-    fn finish_subframe(&self, frame: &VideoCodecFrame) -> Result<gst::FlowSuccess, gst::FlowError> {
-        unsafe {
-            try_from_glib(ffi::gst_video_decoder_finish_subframe(
-                self.as_ref().to_glib_none().0,
-                frame.to_glib_full(),
-            ))
-        }
-    }
-
-    #[cfg(any(feature = "v1_20", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
-    #[doc(alias = "gst_video_decoder_drop_subframe")]
-    fn drop_subframe(&self, frame: &VideoCodecFrame) -> Result<gst::FlowSuccess, gst::FlowError> {
-        unsafe {
-            try_from_glib(ffi::gst_video_decoder_drop_subframe(
-                self.as_ref().to_glib_none().0,
-                frame.to_glib_full(),
-            ))
-        }
-    }
-
-    #[cfg(any(feature = "v1_20", feature = "dox"))]
-    #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_20")))]
-    #[doc(alias = "gst_video_decoder_have_last_subframe")]
-    fn have_last_subframe(
-        &self,
-        frame: &VideoCodecFrame,
-    ) -> Result<gst::FlowSuccess, gst::FlowError> {
-        unsafe {
-            try_from_glib(ffi::gst_video_decoder_have_last_subframe(
-                self.as_ref().to_glib_none().0,
-                frame.to_glib_none().0,
-            ))
-        }
-    }
-
     fn sink_pad(&self) -> &gst::Pad {
         unsafe {
             let elt = &*(self.as_ptr() as *const ffi::GstVideoDecoder);
@@ -450,7 +361,7 @@ macro_rules! video_decoder_error(
             Some($msg),
             Some($debug),
             file!(),
-            module_path!(),
+            $crate::glib::function_name!(),
             line!(),
         )
     }};
@@ -462,7 +373,7 @@ macro_rules! video_decoder_error(
             Some($msg),
             None,
             file!(),
-            module_path!(),
+            $crate::glib::function_name!(),
             line!(),
         )
     }};
@@ -474,7 +385,7 @@ macro_rules! video_decoder_error(
             None,
             Some($debug),
             file!(),
-            module_path!(),
+            $crate::glib::function_name!(),
             line!(),
         )
     }};
@@ -486,7 +397,7 @@ macro_rules! video_decoder_error(
             Some(&format!($($msg)*)),
             Some(&format!($($debug)*)),
             file!(),
-            module_path!(),
+            $crate::glib::function_name!(),
             line!(),
         )
     }};
@@ -498,7 +409,7 @@ macro_rules! video_decoder_error(
             Some(&format!($($msg)*)),
             None,
             file!(),
-            module_path!(),
+            $crate::glib::function_name!(),
             line!(),
         )
     }};
@@ -510,7 +421,7 @@ macro_rules! video_decoder_error(
             None,
             Some(&format!($($debug)*)),
             file!(),
-            module_path!(),
+            $crate::glib::function_name!(),
             line!(),
         )
     }};
