@@ -1,13 +1,14 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use crate::utils::HasStreamLock;
-use crate::video_codec_state::{InNegotiation, Readable, VideoCodecState, VideoCodecStateContext};
-use crate::VideoCodecFrame;
-use crate::VideoEncoder;
-use glib::prelude::*;
-use glib::translate::*;
-use std::mem;
-use std::ptr;
+use std::{mem, ptr};
+
+use glib::{prelude::*, translate::*};
+
+use crate::{
+    utils::HasStreamLock,
+    video_codec_state::{InNegotiation, Readable, VideoCodecState, VideoCodecStateContext},
+    VideoCodecFrame, VideoEncoder,
+};
 
 pub trait VideoEncoderExtManual: 'static {
     #[doc(alias = "gst_video_encoder_allocate_output_frame")]
@@ -31,12 +32,6 @@ pub trait VideoEncoderExtManual: 'static {
     #[doc(alias = "gst_video_encoder_get_allocator")]
     fn allocator(&self) -> (Option<gst::Allocator>, gst::AllocationParams);
 
-    #[doc(alias = "gst_video_encoder_finish_frame")]
-    fn finish_frame(
-        &self,
-        frame: Option<VideoCodecFrame>,
-    ) -> Result<gst::FlowSuccess, gst::FlowError>;
-
     #[cfg(any(feature = "v1_18", feature = "dox"))]
     #[cfg_attr(feature = "dox", doc(cfg(feature = "v1_18")))]
     #[doc(alias = "gst_video_encoder_finish_subframe")]
@@ -68,6 +63,9 @@ pub trait VideoEncoderExtManual: 'static {
         output_state: VideoCodecState<'a, InNegotiation<'a>>,
     ) -> Result<(), gst::FlowError>;
 
+    #[doc(alias = "gst_video_encoder_set_headers")]
+    fn set_headers(&self, headers: impl IntoIterator<Item = gst::Buffer>);
+
     fn sink_pad(&self) -> &gst::Pad;
 
     fn src_pad(&self) -> &gst::Pad;
@@ -91,25 +89,13 @@ impl<O: IsA<VideoEncoder>> VideoEncoderExtManual for O {
     fn allocator(&self) -> (Option<gst::Allocator>, gst::AllocationParams) {
         unsafe {
             let mut allocator = ptr::null_mut();
-            let mut params = mem::zeroed();
+            let mut params = mem::MaybeUninit::uninit();
             ffi::gst_video_encoder_get_allocator(
                 self.as_ref().to_glib_none().0,
                 &mut allocator,
-                &mut params,
+                params.as_mut_ptr(),
             );
-            (from_glib_full(allocator), params.into())
-        }
-    }
-
-    fn finish_frame(
-        &self,
-        frame: Option<VideoCodecFrame>,
-    ) -> Result<gst::FlowSuccess, gst::FlowError> {
-        unsafe {
-            try_from_glib(ffi::gst_video_encoder_finish_frame(
-                self.as_ref().to_glib_none().0,
-                frame.map(|f| f.into_glib_ptr()).unwrap_or(ptr::null_mut()),
-            ))
+            (from_glib_full(allocator), params.assume_init().into())
         }
     }
 
@@ -253,6 +239,18 @@ impl<O: IsA<VideoEncoder>> VideoEncoderExtManual for O {
         }
     }
 
+    fn set_headers(&self, headers: impl IntoIterator<Item = gst::Buffer>) {
+        unsafe {
+            ffi::gst_video_encoder_set_headers(
+                self.as_ref().to_glib_none().0,
+                headers
+                    .into_iter()
+                    .collect::<glib::List<_>>()
+                    .into_glib_ptr(),
+            );
+        }
+    }
+
     fn sink_pad(&self) -> &gst::Pad {
         unsafe {
             let elt = &*(self.as_ptr() as *const ffi::GstVideoEncoder);
