@@ -1,22 +1,56 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
+use std::{cmp, fmt, mem, str};
+
 use glib::translate::*;
 use gst::prelude::*;
-use std::cmp;
-use std::fmt;
-use std::mem;
-use std::ptr;
-use std::str;
 
-use crate::VideoTimeCodeFlags;
-use crate::VideoTimeCodeInterval;
+use crate::{VideoTimeCodeFlags, VideoTimeCodeInterval};
 
-#[doc(alias = "GstVideoTimeCode")]
-#[repr(transparent)]
-pub struct VideoTimeCode(ffi::GstVideoTimeCode);
-#[doc(alias = "GstVideoTimeCode")]
-#[repr(transparent)]
-pub struct ValidVideoTimeCode(ffi::GstVideoTimeCode);
+glib::wrapper! {
+    #[doc(alias = "GstVideoTimeCode")]
+    pub struct VideoTimeCode(BoxedInline<ffi::GstVideoTimeCode>);
+
+    match fn {
+        copy => |ptr| ffi::gst_video_time_code_copy(ptr),
+        free => |ptr| ffi::gst_video_time_code_free(ptr),
+        init => |_ptr| (),
+        copy_into => |dest, src| {
+            *dest = *src;
+            if !(*dest).config.latest_daily_jam.is_null() {
+                glib::ffi::g_date_time_ref((*dest).config.latest_daily_jam);
+            }
+        },
+        clear => |ptr| {
+            if !(*ptr).config.latest_daily_jam.is_null() {
+                glib::ffi::g_date_time_unref((*ptr).config.latest_daily_jam);
+            }
+        },
+        type_ => || ffi::gst_video_time_code_get_type(),
+    }
+}
+
+glib::wrapper! {
+    #[doc(alias = "GstVideoTimeCode")]
+    pub struct ValidVideoTimeCode(BoxedInline<ffi::GstVideoTimeCode>);
+
+    match fn {
+        copy => |ptr| ffi::gst_video_time_code_copy(ptr),
+        free => |ptr| ffi::gst_video_time_code_free(ptr),
+        init => |_ptr| (),
+        copy_into => |dest, src| {
+            *dest = *src;
+            if !(*dest).config.latest_daily_jam.is_null() {
+                glib::ffi::g_date_time_ref((*dest).config.latest_daily_jam);
+            }
+        },
+        clear => |ptr| {
+            if !(*ptr).config.latest_daily_jam.is_null() {
+                glib::ffi::g_date_time_unref((*ptr).config.latest_daily_jam);
+            }
+        },
+    }
+}
 
 impl VideoTimeCode {
     pub fn new_empty() -> Self {
@@ -24,7 +58,9 @@ impl VideoTimeCode {
         unsafe {
             let mut v = mem::MaybeUninit::zeroed();
             ffi::gst_video_time_code_clear(v.as_mut_ptr());
-            Self(v.assume_init())
+            Self {
+                inner: v.assume_init(),
+            }
         }
     }
 
@@ -41,7 +77,7 @@ impl VideoTimeCode {
     ) -> Self {
         assert_initialized_main_thread!();
         unsafe {
-            let mut v = mem::MaybeUninit::zeroed();
+            let mut v = mem::MaybeUninit::uninit();
             ffi::gst_video_time_code_init(
                 v.as_mut_ptr(),
                 fps.numer() as u32,
@@ -55,7 +91,9 @@ impl VideoTimeCode {
                 field_count,
             );
 
-            Self(v.assume_init())
+            Self {
+                inner: v.assume_init(),
+            }
         }
     }
 
@@ -84,7 +122,9 @@ impl VideoTimeCode {
             if res == glib::ffi::GFALSE {
                 Err(glib::bool_error!("Failed to init video time code"))
             } else {
-                Ok(Self(v.assume_init()))
+                Ok(Self {
+                    inner: v.assume_init(),
+                })
             }
         }
     }
@@ -94,36 +134,43 @@ impl VideoTimeCode {
         unsafe { from_glib(ffi::gst_video_time_code_is_valid(self.to_glib_none().0)) }
     }
 
+    #[inline]
     pub fn set_fps(&mut self, fps: gst::Fraction) {
-        self.0.config.fps_n = fps.numer() as u32;
-        self.0.config.fps_d = fps.denom() as u32;
+        self.inner.config.fps_n = fps.numer() as u32;
+        self.inner.config.fps_d = fps.denom() as u32;
     }
 
+    #[inline]
     pub fn set_flags(&mut self, flags: VideoTimeCodeFlags) {
-        self.0.config.flags = flags.into_glib()
+        self.inner.config.flags = flags.into_glib()
     }
 
+    #[inline]
     pub fn set_hours(&mut self, hours: u32) {
-        self.0.hours = hours
+        self.inner.hours = hours
     }
 
+    #[inline]
     pub fn set_minutes(&mut self, minutes: u32) {
         assert!(minutes < 60);
-        self.0.minutes = minutes
+        self.inner.minutes = minutes
     }
 
+    #[inline]
     pub fn set_seconds(&mut self, seconds: u32) {
         assert!(seconds < 60);
-        self.0.seconds = seconds
+        self.inner.seconds = seconds
     }
 
+    #[inline]
     pub fn set_frames(&mut self, frames: u32) {
-        self.0.frames = frames
+        self.inner.frames = frames
     }
 
+    #[inline]
     pub fn set_field_count(&mut self, field_count: u32) {
         assert!(field_count <= 2);
-        self.0.field_count = field_count
+        self.inner.field_count = field_count
     }
 }
 
@@ -137,7 +184,7 @@ impl TryFrom<VideoTimeCode> for ValidVideoTimeCode {
             // from running as we don't move v.0 out here but copy it.
             // GstVideoTimeCode implements Copy.
             let v = mem::ManuallyDrop::new(v);
-            Ok(Self(v.0))
+            Ok(Self { inner: v.inner })
         } else {
             Err(v)
         }
@@ -156,7 +203,7 @@ impl ValidVideoTimeCode {
         frames: u32,
         field_count: u32,
     ) -> Result<Self, glib::error::BoolError> {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         let tc = VideoTimeCode::new(
             fps,
             latest_daily_jam,
@@ -186,7 +233,6 @@ impl ValidVideoTimeCode {
 
     #[doc(alias = "gst_video_time_code_add_frames")]
     pub fn add_frames(&mut self, frames: i64) {
-        skip_assert_initialized!();
         unsafe {
             ffi::gst_video_time_code_add_frames(self.to_glib_none_mut().0, frames);
         }
@@ -249,68 +295,58 @@ impl ValidVideoTimeCode {
 macro_rules! generic_impl {
     ($name:ident) => {
         impl $name {
+            #[inline]
             pub fn hours(&self) -> u32 {
-                self.0.hours
+                self.inner.hours
             }
 
+            #[inline]
             pub fn minutes(&self) -> u32 {
-                self.0.minutes
+                self.inner.minutes
             }
 
+            #[inline]
             pub fn seconds(&self) -> u32 {
-                self.0.seconds
+                self.inner.seconds
             }
 
+            #[inline]
             pub fn frames(&self) -> u32 {
-                self.0.frames
+                self.inner.frames
             }
 
+            #[inline]
             pub fn field_count(&self) -> u32 {
-                self.0.field_count
+                self.inner.field_count
             }
 
+            #[inline]
             pub fn fps(&self) -> gst::Fraction {
-                (self.0.config.fps_n as i32, self.0.config.fps_d as i32).into()
+                (
+                    self.inner.config.fps_n as i32,
+                    self.inner.config.fps_d as i32,
+                )
+                    .into()
             }
 
+            #[inline]
             pub fn flags(&self) -> VideoTimeCodeFlags {
-                unsafe { from_glib(self.0.config.flags) }
+                unsafe { from_glib(self.inner.config.flags) }
             }
 
+            #[inline]
             pub fn latest_daily_jam(&self) -> Option<glib::DateTime> {
-                unsafe { from_glib_none(self.0.config.latest_daily_jam) }
+                unsafe { from_glib_none(self.inner.config.latest_daily_jam) }
             }
 
-            pub fn set_latest_daily_jam(&mut self, latest_daily_jam: Option<&glib::DateTime>) {
+            #[inline]
+            pub fn set_latest_daily_jam(&mut self, latest_daily_jam: Option<glib::DateTime>) {
                 unsafe {
-                    if !self.0.config.latest_daily_jam.is_null() {
-                        glib::ffi::g_date_time_unref(self.0.config.latest_daily_jam);
+                    if !self.inner.config.latest_daily_jam.is_null() {
+                        glib::ffi::g_date_time_unref(self.inner.config.latest_daily_jam);
                     }
 
-                    self.0.config.latest_daily_jam = latest_daily_jam.to_glib_full()
-                }
-            }
-        }
-
-        impl Clone for $name {
-            fn clone(&self) -> Self {
-                unsafe {
-                    let v = self.0;
-                    if !v.config.latest_daily_jam.is_null() {
-                        glib::ffi::g_date_time_ref(v.config.latest_daily_jam);
-                    }
-
-                    $name(v)
-                }
-            }
-        }
-
-        impl Drop for $name {
-            fn drop(&mut self) {
-                unsafe {
-                    if !self.0.config.latest_daily_jam.is_null() {
-                        glib::ffi::g_date_time_unref(self.0.config.latest_daily_jam);
-                    }
+                    self.inner.config.latest_daily_jam = latest_daily_jam.into_glib_ptr();
                 }
             }
         }
@@ -344,153 +380,60 @@ macro_rules! generic_impl {
 
         unsafe impl Send for $name {}
         unsafe impl Sync for $name {}
-
-        #[doc(hidden)]
-        impl GlibPtrDefault for $name {
-            type GlibType = *mut ffi::GstVideoTimeCode;
-        }
-
-        #[doc(hidden)]
-        impl<'a> ToGlibPtr<'a, *const ffi::GstVideoTimeCode> for $name {
-            type Storage = &'a Self;
-
-            #[inline]
-            fn to_glib_none(&'a self) -> Stash<'a, *const ffi::GstVideoTimeCode, Self> {
-                Stash(&self.0 as *const _, self)
-            }
-
-            #[inline]
-            fn to_glib_full(&self) -> *const ffi::GstVideoTimeCode {
-                unsafe { ffi::gst_video_time_code_copy(&self.0 as *const _) }
-            }
-        }
-
-        #[doc(hidden)]
-        impl<'a> ToGlibPtrMut<'a, *mut ffi::GstVideoTimeCode> for $name {
-            type Storage = &'a mut Self;
-
-            #[inline]
-            fn to_glib_none_mut(&'a mut self) -> StashMut<'a, *mut ffi::GstVideoTimeCode, Self> {
-                let ptr = &mut self.0 as *mut _;
-                StashMut(ptr, self)
-            }
-        }
-
-        #[doc(hidden)]
-        impl FromGlibPtrNone<*mut ffi::GstVideoTimeCode> for $name {
-            #[inline]
-            unsafe fn from_glib_none(ptr: *mut ffi::GstVideoTimeCode) -> Self {
-                assert!(!ptr.is_null());
-                let v = ptr::read(ptr);
-                if !v.config.latest_daily_jam.is_null() {
-                    glib::ffi::g_date_time_ref(v.config.latest_daily_jam);
-                }
-
-                $name(v)
-            }
-        }
-
-        #[doc(hidden)]
-        impl FromGlibPtrNone<*const ffi::GstVideoTimeCode> for $name {
-            #[inline]
-            unsafe fn from_glib_none(ptr: *const ffi::GstVideoTimeCode) -> Self {
-                assert!(!ptr.is_null());
-                let v = ptr::read(ptr);
-                if !v.config.latest_daily_jam.is_null() {
-                    glib::ffi::g_date_time_ref(v.config.latest_daily_jam);
-                }
-
-                $name(v)
-            }
-        }
-
-        #[doc(hidden)]
-        impl FromGlibPtrFull<*mut ffi::GstVideoTimeCode> for $name {
-            #[inline]
-            unsafe fn from_glib_full(ptr: *mut ffi::GstVideoTimeCode) -> Self {
-                assert!(!ptr.is_null());
-                let v = ptr::read(ptr);
-                if !v.config.latest_daily_jam.is_null() {
-                    glib::ffi::g_date_time_ref(v.config.latest_daily_jam);
-                }
-                ffi::gst_video_time_code_free(ptr);
-
-                $name(v)
-            }
-        }
-
-        #[doc(hidden)]
-        impl FromGlibPtrBorrow<*mut ffi::GstVideoTimeCode> for $name {
-            #[inline]
-            unsafe fn from_glib_borrow(ptr: *mut ffi::GstVideoTimeCode) -> Borrowed<Self> {
-                assert!(!ptr.is_null());
-                let v = ptr::read(ptr);
-
-                Borrowed::new($name(v))
-            }
-        }
-
-        impl StaticType for $name {
-            fn static_type() -> glib::Type {
-                unsafe { from_glib(ffi::gst_video_time_code_get_type()) }
-            }
-        }
-
-        impl glib::value::ValueType for $name {
-            type Type = Self;
-        }
-
-        impl glib::value::ValueTypeOptional for $name {}
-
-        #[doc(hidden)]
-        unsafe impl<'a> glib::value::FromValue<'a> for $name {
-            type Checker = glib::value::GenericValueTypeOrNoneChecker<Self>;
-
-            unsafe fn from_value(value: &'a glib::Value) -> Self {
-                skip_assert_initialized!();
-                from_glib_none(glib::gobject_ffi::g_value_get_boxed(value.to_glib_none().0)
-                    as *mut ffi::GstVideoTimeCode)
-            }
-        }
-
-        #[doc(hidden)]
-        impl glib::value::ToValue for $name {
-            fn to_value(&self) -> glib::Value {
-                let mut value = glib::Value::for_value_type::<Self>();
-                unsafe {
-                    glib::gobject_ffi::g_value_set_boxed(
-                        value.to_glib_none_mut().0,
-                        self.to_glib_none().0 as *mut _,
-                    )
-                }
-                value
-            }
-
-            fn value_type(&self) -> glib::Type {
-                Self::static_type()
-            }
-        }
-
-        #[doc(hidden)]
-        impl glib::value::ToValueOptional for $name {
-            fn to_value_optional(s: Option<&Self>) -> glib::Value {
-                skip_assert_initialized!();
-                let mut value = glib::Value::for_value_type::<Self>();
-                unsafe {
-                    glib::gobject_ffi::g_value_set_boxed(
-                        value.to_glib_none_mut().0,
-                        s.to_glib_none().0 as *mut _,
-                    )
-                }
-                value
-            }
-        }
     };
 }
 
 generic_impl!(VideoTimeCode);
 generic_impl!(ValidVideoTimeCode);
 
+impl StaticType for ValidVideoTimeCode {
+    #[inline]
+    fn static_type() -> glib::Type {
+        unsafe { from_glib(ffi::gst_video_time_code_get_type()) }
+    }
+}
+
+#[doc(hidden)]
+impl glib::value::ToValue for ValidVideoTimeCode {
+    fn to_value(&self) -> glib::Value {
+        let mut value = glib::Value::for_value_type::<VideoTimeCode>();
+        unsafe {
+            glib::gobject_ffi::g_value_set_boxed(
+                value.to_glib_none_mut().0,
+                self.to_glib_none().0 as *mut _,
+            )
+        }
+        value
+    }
+
+    fn value_type(&self) -> glib::Type {
+        Self::static_type()
+    }
+}
+
+#[doc(hidden)]
+impl glib::value::ToValueOptional for ValidVideoTimeCode {
+    fn to_value_optional(s: Option<&Self>) -> glib::Value {
+        skip_assert_initialized!();
+        let mut value = glib::Value::for_value_type::<VideoTimeCode>();
+        unsafe {
+            glib::gobject_ffi::g_value_set_boxed(
+                value.to_glib_none_mut().0,
+                s.to_glib_none().0 as *mut _,
+            )
+        }
+        value
+    }
+}
+
+#[doc(hidden)]
+impl From<ValidVideoTimeCode> for glib::Value {
+    fn from(v: ValidVideoTimeCode) -> glib::Value {
+        skip_assert_initialized!();
+        glib::value::ToValue::to_value(&v)
+    }
+}
+
 impl str::FromStr for VideoTimeCode {
     type Err = glib::error::BoolError;
 
@@ -530,13 +473,14 @@ impl Ord for ValidVideoTimeCode {
 }
 
 impl From<ValidVideoTimeCode> for VideoTimeCode {
+    #[inline]
     fn from(v: ValidVideoTimeCode) -> Self {
         skip_assert_initialized!();
         // Use ManuallyDrop here to prevent the Drop impl of VideoTimeCode
         // from running as we don't move v.0 out here but copy it.
         // GstVideoTimeCode implements Copy.
         let v = mem::ManuallyDrop::new(v);
-        Self(v.0)
+        Self { inner: v.inner }
     }
 }
 
@@ -565,10 +509,12 @@ impl VideoTimeCodeMeta {
     }
 
     #[doc(alias = "get_tc")]
+    #[inline]
     pub fn tc(&self) -> ValidVideoTimeCode {
         unsafe { ValidVideoTimeCode::from_glib_none(&self.0.tc as *const _) }
     }
 
+    #[inline]
     pub fn set_tc(&mut self, tc: ValidVideoTimeCode) {
         #![allow(clippy::cast_ptr_alignment)]
         unsafe {
@@ -577,7 +523,7 @@ impl VideoTimeCodeMeta {
             // from running as we don't move tc.0 out here but copy it.
             // GstVideoTimeCode implements Copy.
             let tc = mem::ManuallyDrop::new(tc);
-            self.0.tc = tc.0;
+            self.0.tc = tc.inner;
         }
     }
 }
@@ -586,6 +532,7 @@ unsafe impl MetaAPI for VideoTimeCodeMeta {
     type GstType = ffi::GstVideoTimeCodeMeta;
 
     #[doc(alias = "gst_video_time_code_meta_api_get_type")]
+    #[inline]
     fn meta_api() -> glib::Type {
         unsafe { from_glib(ffi::gst_video_time_code_meta_api_get_type()) }
     }
